@@ -35,7 +35,9 @@ import unipath
 from unipath import pathof
 import unicodedata
 
-_launcher_version=20151024
+_launcher_version=20190218
+
+_PKG_VER = re.compile(r'''<\s*package[^>]*version\s*=\s*["']([^'"]*)['"][^>]*>''',re.IGNORECASE)
 
 # Wrapper Class is used to peform record keeping for Sigil.  It keeps track of modified,
 # added, and deleted files while providing some degree of protection against files under
@@ -47,50 +49,62 @@ _guide_types = ['cover','title-page','toc','index','glossary','acknowledgements'
                 'epigraph','foreward','loi','lot','notes','preface','text']
 
 ext_mime_map = {
-                '.jpg'  : 'image/jpeg',
-                '.jpeg' : 'image/jpeg',
-                '.png'  : 'image/png',
-                '.gif'  : 'image/gif',
-                '.svg'  : 'image/svg+xml',
-                '.xhtml': 'application/xhtml+xml',
-                '.html' : 'application/xhtml+xml',
-                '.ttf'  : 'application/x-font-ttf',
-                '.otf'  : 'application/x-font-opentype',
-                '.woff' : 'application/font-woff',
-                '.mp3'  : 'audio/mpeg',
-                '.mp4'  : 'video/mp4',
-                '.css'  : 'text/css',
-                '.ncx'  : 'application/x-dtbncx+xml',
-                '.xml'  : 'application/oebs-page-map+xml',
-                '.opf'  : 'application/oebps-package+xml',
-                '.smil' : 'application/smil+xml',
-                '.pls'  : 'application/pls-xml',
-                '.js'   : 'text/javascript',
-                '.epub' : 'application/epub+zip',
-                #'.js'   : 'application/javascript',
-                #'.otf'  : 'application/vnd.ms-opentype',
+                '.jpg'   : 'image/jpeg',
+                '.jpeg'  : 'image/jpeg',
+                '.png'   : 'image/png',
+                '.gif'   : 'image/gif',
+                '.svg'   : 'image/svg+xml',
+                '.xhtml' : 'application/xhtml+xml',
+                '.html'  : 'application/xhtml+xml',
+                '.htm'   : 'application/xhtml+xml',
+                '.otf'   : 'application/vnd.ms-opentype',
+                #'.otf'  : 'application/x-font-opentype',
+                #'.otf'  : 'application/font-sfnt',
+                '.ttf'   : 'application/x-font-ttf',
+                '.woff'  : 'application/font-woff',
+                '.woff2' : 'font/woff2',
+                '.mp3'   : 'audio/mpeg',
+                '.m4a'   : 'audio/mp4',
+                '.mp4'   : 'video/mp4',
+                '.m4v'   : 'video/mp4',
+                '.css'   : 'text/css',
+                '.ncx'   : 'application/x-dtbncx+xml',
+                '.xml'   : 'application/oebs-page-map+xml',
+                '.opf'   : 'application/oebps-package+xml',
+                '.smil'  : 'application/smil+xml',
+                '.pls'   : 'application/pls+xml',
+                '.xpgt'  : 'application/adobe-page-template+xml',
+                #'.js'   : 'text/javascript',
+                '.js'    : 'application/javascript',
+                '.epub'  : 'application/epub+zip',
                 }
 
 mime_base_map = {
-                'image/jpeg'                    : 'Images',
-                'image/png'                     : 'Images',
-                'image/gif'                     : 'Images',
-                'image/svg+xml'                 : 'Images',
-                'application/xhtml+xml'         : 'Text',
-                'application/x-font-ttf'        : 'Fonts',
-                'application/x-font-opentype'   : 'Fonts',
-                'application/vnd.ms-opentype'   : 'Fonts',
-                'application/font-woff'         : 'Fonts',
-                'audio/mpeg'                    : 'Audio',
-                'audio/mp3'                     : 'Audio',
-                'audio/mp4'                     : 'Audio',
-                'video/mp4'                     : 'Video',
-                'text/css'                      : 'Styles',
-                'application/x-dtbncx+xml'      : '',
-                'application/oebps-package+xml' : '',
-                'application/oebs-page-map+xml' : 'Misc',
-                'application/smil+xml'          : 'Misc',
-                'application/pls-xml'           : 'Misc',
+                'image/jpeg'                              : 'Images',
+                'image/png'                               : 'Images',
+                'image/gif'                               : 'Images',
+                'image/svg+xml'                           : 'Images',
+                'application/xhtml+xml'                   : 'Text',
+                'application/x-font-ttf'                  : 'Fonts',
+                'application/x-font-opentype'             : 'Fonts',
+                'application/vnd.ms-opentype'             : 'Fonts',
+                'application/font-woff'                   : 'Fonts',
+                'application/font-sfnt'                   : 'Fonts',
+                'font/woff2'                              : 'Fonts',
+                'audio/mpeg'                              : 'Audio',
+                'audio/mp3'                               : 'Audio',
+                'audio/mp4'                               : 'Audio',
+                'video/mp4'                               : 'Video',
+                'text/css'                                : 'Styles',
+                'application/x-dtbncx+xml'                : '',
+                'application/oebps-package+xml'           : '',
+                'application/oebs-page-map+xml'           : 'Misc',
+                'application/smil+xml'                    : 'Misc',
+                'application/adobe-page-template+xml'     : 'Misc',
+                'application/vnd.adobe-page-template+xml' : 'Misc',
+                'text/javascript'                         : 'Misc',
+                'application/javascript'                  : 'Misc',
+                'application/pls+xml'                     : 'Misc',
                 }
 
 
@@ -100,6 +114,20 @@ PROTECTED_FILES = [
     'OEBPS/content.opf'
 ]
 
+TEXT_MIMETYPES = [
+                'image/svg+xml',
+                'application/xhtml+xml',
+                'text/css',
+                'application/x-dtbncx+xml',
+                'application/oebps-package+xml',
+                'application/oebs-page-map+xml',
+                'application/smil+xml',
+                'application/adobe-page-template+xml',
+                'application/vnd.adobe-page-template+xml',
+                'text/javascript',
+                'application/javascript'
+                'application/pls+xml'
+]
 
 class WrapperException(Exception):
     pass
@@ -117,6 +145,16 @@ class Wrapper(object):
         # initialize the sigil cofiguration info passed in outdir with sigil.cfg
         self.appdir = None
         self.usrsupdir = None
+        # Location of directory containing hunspell dictionaries on Linux
+        self.linux_hunspell_dict_dirs = []
+        # Sigil interface language code
+        self.sigil_ui_lang = None
+        # Default Sigil spell check dictionary
+        self.sigil_spellcheck_lang = None
+        # status of epub inside Sigil (isDirty) and CurrentFilePath of current epub file
+        self.epub_isDirty = False
+        self.epub_filepath = ""
+        # File selected in Sigil's Book Browser
         self.selected = []
         cfg = ''
         with open(os.path.join(self.outdir, 'sigil.cfg'), 'rb') as f:
@@ -126,6 +164,12 @@ class Wrapper(object):
         if len(cfg_lst) >= 2:
             self.appdir = cfg_lst.pop(0)
             self.usrsupdir = cfg_lst.pop(0)
+            if not sys.platform.startswith('darwin') and not sys.platform.startswith('win'):
+                self.linux_hunspell_dict_dirs = cfg_lst.pop(0).split(":")
+            self.sigil_ui_lang = cfg_lst.pop(0)
+            self.sigil_spellcheck_lang = cfg_lst.pop(0)
+            self.epub_isDirty = (cfg_lst.pop(0) == "True")
+            self.epub_filepath = cfg_lst.pop(0)
             self.selected = cfg_lst
         os.environ['SigilGumboLibPath'] = self.get_gumbo_path()
 
@@ -134,10 +178,14 @@ class Wrapper(object):
         self.id_to_mime = {}
         self.href_to_id = {}
         self.id_to_props = {}
+        self.id_to_fall = {}
+        self.id_to_over = {}
         self.spine_ppd = None
         self.spine = []
         self.guide = []
+        self.bindings = []
         self.package_tag = None
+        self.epub_version = None
         # self.metadata_attr = None
         # self.metadata = []
         self.metadataxml = ''
@@ -149,15 +197,20 @@ class Wrapper(object):
             self.id_to_mime = op.get_manifest_id_to_mime_dict().copy()
             self.href_to_id = op.get_href_to_manifest_id_dict().copy()
             self.id_to_props = op.get_manifest_id_to_properties_dict().copy()
+            self.id_to_fall = op.get_manifest_id_to_fallback_dict().copy()
+            self.id_to_over = op.get_manifest_id_to_overlay_dict().copy()
             self.spine_ppd = op.get_spine_ppd()
             self.spine = op.get_spine()
             self.guide = op.get_guide()
             self.package_tag = op.get_package_tag()
+            self.epub_version = op.get_epub_version()
+            self.bindings = op.get_bindings()
             # self.metadata = op.get_metadata()
             # self.metadata_attr = op.get_metadata_attr()
             self.metadataxml = op.get_metadataxml()
         self.other = []  # non-manifest file information
         self.id_to_filepath = {}
+        self.book_href_to_filepath = {}
         self.modified = {}
         self.added = []
         self.deleted = []
@@ -176,7 +229,7 @@ class Wrapper(object):
                 id = self.href_to_id.get(href,None)
             if id is None:
                 self.other.append(book_href)
-                self.id_to_filepath[book_href] = filepath
+                self.book_href_to_filepath[book_href] = filepath
             else:
                 self.id_to_filepath[id] = filepath
 
@@ -184,10 +237,13 @@ class Wrapper(object):
         global _launcher_version
         return _launcher_version
 
+    def getepubversion(self):
+        return self.epub_version
 
     # utility routine to get mime from href
     def getmime(self,  href):
         href = unicode_str(href)
+        href = unquoteurl(href)
         filename = os.path.basename(href)
         ext = os.path.splitext(filename)[1]
         ext = ext.lower()
@@ -201,7 +257,7 @@ class Wrapper(object):
 
     def build_manifest_xml(self):
         manout = []
-        manout.append('<manifest>\n')
+        manout.append('  <manifest>\n')
         for id in sorted(self.id_to_mime):
             href = quoteurl(self.id_to_href[id])
             mime = self.id_to_mime[id]
@@ -209,8 +265,16 @@ class Wrapper(object):
             properties = self.id_to_props[id]
             if properties is not None:
                 props = ' properties="%s"' % properties
-            manout.append('<item id="%s" href="%s" media-type="%s"%s />\n' % (id, href, mime, props))
-        manout.append('</manifest>\n')
+            fall = ''
+            fallback = self.id_to_fall[id]
+            if fallback is not None:
+                fall = ' fallback="%s"' % fallback
+            over = ''
+            overlay = self.id_to_over[id]
+            if overlay is not None:
+                over = ' media-overlay="%s"' % overlay
+            manout.append('    <item id="%s" href="%s" media-type="%s"%s%s%s />\n' % (id, href, mime, props, fall, over))
+        manout.append('  </manifest>\n')
         return "".join(manout)
 
     def build_spine_xml(self):
@@ -226,7 +290,7 @@ class Wrapper(object):
         pagemapid = self.getpagemapid()
         if pagemapid is not None:
             map = ' page-map="%s"' % pagemapid
-        spineout.append('<spine%s%s%s>\n' %(ppd, ncx, map))
+        spineout.append('  <spine%s%s%s>\n' %(ppd, ncx, map))
         for (id, linear, properties) in self.spine:
             lin = ''
             if linear is not None:
@@ -234,19 +298,29 @@ class Wrapper(object):
             props = ''
             if properties is not None:
                 props = ' properties="%s"' % properties
-            spineout.append('<itemref idref="%s"%s%s/>\n' % (id, lin, props))
-        spineout.append('</spine>\n')
+            spineout.append('    <itemref idref="%s"%s%s/>\n' % (id, lin, props))
+        spineout.append('  </spine>\n')
         return "".join(spineout)
 
     def build_guide_xml(self):
         guideout = []
-        guideout.append('<guide>\n')
-        for (type, title, href) in self.guide:
-            href = quoteurl(href)
-            guideout.append('<reference type="%s" href="%s" title="%s"/>\n' % (type, href, title))
-        guideout.append('</guide>\n')
+        if len(self.guide) > 0:
+            guideout.append('  <guide>\n')
+            for (type, title, href) in self.guide:
+                href = quoteurl(href)
+                guideout.append('    <reference type="%s" href="%s" title="%s"/>\n' % (type, href, title))
+            guideout.append('  </guide>\n')
         return "".join(guideout)
 
+    def build_bindings_xml(self):
+        bindout = []
+        if len(self.bindings) > 0 and self.epub_version.startswith('3'):
+            bindout.append('  <bindings>\n')
+            for (mtype, handler) in self.bindings:
+                bindout.append('    <mediaType media-type="%s" handler="%s"/>\n' % (mtype, handler))
+            bindout.append('  </bindings>\n')
+        return "".join(bindout)
+
     def build_opf(self):
         data = []
         data.append('<?xml version="1.0" encoding="utf-8" standalone="yes"?>\n')
@@ -255,6 +329,7 @@ class Wrapper(object):
         data.append(self.build_manifest_xml())
         data.append(self.build_spine_xml())
         data.append(self.build_guide_xml())
+        data.append(self.build_bindings_xml())
         data.append('</package>\n')
         return "".join(data)
 
@@ -333,6 +408,24 @@ class Wrapper(object):
         self.spine = spine
         self.modified['OEBPS/content.opf'] = 'file'
 
+    def getbindings_epub3(self):
+        return self.bindings
+
+    def setbindings_epub3(self, new_bindings):
+        bindings = []
+        for (mtype, handler) in new_bindings:
+            mtype = unicode_str(mtype)
+            handler = unicode_str(handler)
+            if mtype is None or mtype == "":
+                continue
+            if handler is None or handler == "":
+                continue
+            if handler not in self.id_to_href:
+                raise WrapperException('Handler not in Manifest')
+            bindings.append((mtype, handler))
+        self.bindings = bindings
+        self.modified['OEBPS/content.opf'] = 'file'
+
     def spine_insert_before(self, pos, sid, linear, properties=None):
         sid = unicode_str(sid)
         linear = unicode_str(linear)
@@ -345,7 +438,7 @@ class Wrapper(object):
         if pos == 0:
             self.spine = [(sid, linear, properties)] + self.spine
         elif pos == -1 or pos >= n:
-            self.spine = self.spine.append((sid, linear, properties))
+            self.spine.append((sid, linear, properties))
         else:
             self.spine = self.spine[0:pos] + [(sid, linear, properties)] + self.spine[pos:]
         self.modified['OEBPS/content.opf'] = 'file'
@@ -390,6 +483,7 @@ class Wrapper(object):
             type = unicode_str(type)
             title = unicode_str(title)
             href = unicode_str(href)
+            href = unquoteurl(href)
             if type not in _guide_types:
                 type = "other." + type
             if title is None:
@@ -417,7 +511,14 @@ class Wrapper(object):
         return self.package_tag
 
     def setpackagetag(self, new_packagetag):
-        self.package_tag = unicode_str(new_packagetag)
+        pkgtag = unicode_str(new_packagetag)
+        version = ""
+        mo = _PKG_VER.search(pkgtag)
+        if mo:
+            version = mo.group(1)
+        if version != self.epub_version:
+            raise WrapperException('Illegal to change the package version attribute')
+        self.package_tag = pkgtag
         self.modified['OEBPS/content.opf'] = 'file'
 
 
@@ -441,7 +542,7 @@ class Wrapper(object):
         with open(filepath,'rb') as fp:
             data = fp.read()
         mime = self.id_to_mime.get(id,'')
-        if mime.endswith('+xml'):
+        if mime in TEXT_MIMETYPES:
             data = unicode_str(data)
         return data
 
@@ -457,13 +558,13 @@ class Wrapper(object):
         base = os.path.dirname(filepath)
         if not unipath.exists(base):
             os.makedirs(pathof(base))
-        if mime.endswith('+xml') or isinstance(data, text_type):
+        if mime in TEXT_MIMETYPES or isinstance(data, text_type):
             data = utf8_str(data)
         with open(filepath,'wb') as fp:
             fp.write(data)
         self.modified[id] = 'file'
 
-    def addfile(self, uniqueid, basename, data, mime=None, properties=None):
+    def addfile(self, uniqueid, basename, data, mime=None, properties=None, fallback=None, overlay=None):
         uniqueid = unicode_str(uniqueid)
         basename = unicode_str(basename)
         mime = unicode_str(mime)
@@ -492,13 +593,15 @@ class Wrapper(object):
         base = os.path.dirname(filepath)
         if not unipath.exists(base):
             os.makedirs(base)
-        if mime.endswith('+xml') or isinstance(data, text_type):
+        if mime in TEXT_MIMETYPES or isinstance(data, text_type):
             data = utf8_str(data)
         with open(filepath,'wb') as fp:
             fp.write(data)
         self.id_to_href[uniqueid] = href
         self.id_to_mime[uniqueid] = mime
         self.id_to_props[uniqueid] = properties
+        self.id_to_fall[uniqueid] = fallback
+        self.id_to_over[uniqueid] = overlay
         self.href_to_id[href] = uniqueid
         self.added.append(uniqueid)
         self.modified['OEBPS/content.opf'] = 'file'
@@ -528,6 +631,8 @@ class Wrapper(object):
         del self.id_to_href[id]
         del self.id_to_mime[id]
         del self.id_to_props[id]
+        del self.id_to_fall[id]
+        del self.id_to_over[id]
         del self.href_to_id[href]
         # remove from spine
         new_spine = []
@@ -544,15 +649,25 @@ class Wrapper(object):
             self.modified['OEBPS/content.opf'] = 'file'
         del self.id_to_filepath[id]
 
-    def set_manifest_epub3_properties(self, id, properties):
+    def set_manifest_epub3_attributes(self, id, properties=None, fallback=None, overlay=None):
         id = unicode_str(id)
         properties = unicode_str(properties)
         if properties is not None and properties == "":
             properties = None
+        fallback = unicode_str(fallback)
+        if fallback is not None and fallback == "":
+            fallback = None
+        overlay = unicode_str(overlay)
+        if overlay is not None and overlay == "":
+            overlay = None
         if id not in self.id_to_props:
             raise WrapperException('Id does not exist in manifest')
         del self.id_to_props[id]
+        del self.id_to_fall[id]
+        del self.id_to_over[id]
         self.id_to_props[id] = properties
+        self.id_to_fall[id] = fallback
+        self.id_to_over[id] = overlay
         self.modified['OEBPS/content.opf'] = 'file'
 
 
@@ -560,20 +675,24 @@ class Wrapper(object):
 
     def map_href_to_id(self, href, ow):
         href = unicode_str(href)
+        href = unquoteurl(href)
         return self.href_to_id.get(href,ow)
 
     def map_basename_to_id(self, basename, ow):
         basename = unicode_str(basename)
         ext = os.path.splitext(basename)[1]
         ext = ext.lower()
-        mime = ext_mime_map.get(ext,None)
+        mime = ext_mime_map.get(ext,"")
         if mime.startswith("audio"):
             base = 'Audio'
         elif mime.startswith("video"):
             base = "Video"
         else:
             base = mime_base_map.get(mime,'Misc')
-        href = base + "/" + basename
+        if base == "":
+            href = basename
+        else:
+            href = base + "/" + basename
         return self.href_to_id.get(href,ow)
 
     def map_id_to_href(self, id, ow):
@@ -588,19 +707,32 @@ class Wrapper(object):
         id = unicode_str(id)
         return self.id_to_props.get(id, ow)
 
+    def map_id_to_fallback(self, id, ow):
+        id = unicode_str(id)
+        return self.id_to_fall.get(id, ow)
+
+    def map_id_to_overlay(self, id, ow):
+        id = unicode_str(id)
+        return self.id_to_over.get(id, ow)
+
 
     # routines to work on ebook files that are not part of an opf manifest
+    # their "id" is actually their unique relative path from book root
 
     def readotherfile(self, book_href):
         id = unicode_str(book_href)
-        if id in self.id_to_href:
+        id = unquoteurl(id)
+        if id is None:
+            raise WrapperException('None is not a valid book href')
+        if id not in self.other and id in self.id_to_href:
             raise WrapperException('Incorrect interface routine - use readfile')
-        # handle special case of trying to read the opf
-        if id is not None and id == "OEBPS/content.opf":
-            return self.build_opf()
-        filepath = self.id_to_filepath.get(id, None)
+        # handle special case of trying to read the opf after it has been modified
+        if id == "OEBPS/content.opf":
+            if id in self.modified:
+                return self.build_opf()
+        filepath = self.book_href_to_filepath.get(id, None)
         if filepath is None:
-            raise WrapperException('book href does not exist')
+            raise WrapperException('Book href does not exist')
         basedir = self.ebook_root
         if id in self.added or id in self.modified:
             basedir = self.outdir
@@ -610,21 +742,24 @@ class Wrapper(object):
         basename = os.path.basename(filepath)
         ext = os.path.splitext(basename)[1]
         ext = ext.lower()
-        mime = ext_mime_map.get(ext,'')
+        mime = ext_mime_map.get(ext,"")
         data = b''
         with open(filepath,'rb') as fp:
             data = fp.read()
-        if mime.endswith('+xml'):
+        if mime in TEXT_MIMETYPES:
             data = unicode_str(data)
         return data
 
     def writeotherfile(self, book_href, data):
         id = unicode_str(book_href)
-        if id in self.id_to_href:
+        id = unquoteurl(id)
+        if id is None:
+            raise WrapperException('None is not a valid book href')
+        if id not in self.other and id in self.id_to_href:
             raise WrapperException('Incorrect interface routine - use writefile')
-        filepath = self.id_to_filepath.get(id, None)
+        filepath = self.book_href_to_filepath.get(id, None)
         if filepath is None:
-            raise WrapperException('book href does not exist')
+            raise WrapperException('Book href does not exist')
         if id in PROTECTED_FILES:
             raise WrapperException('Attempt to modify protected file')
         filepath = os.path.join(self.outdir, filepath)
@@ -639,12 +774,15 @@ class Wrapper(object):
 
     def addotherfile(self, book_href, data) :
         id = unicode_str(book_href)
+        id = unquoteurl(id)
+        if id is None:
+            raise WrapperException('None is not a valid book href')
         if id in self.other:
-            raise WrapperException('book href must be unquie')
+            raise WrapperException('Book href must be unique')
         desired_path = id.replace("/",os.sep)
         filepath = os.path.join(self.outdir,desired_path)
         if unipath.isfile(filepath):
-            raise WrapperException('desired path already exists')
+            raise WrapperException('Desired path already exists')
         base = os.path.dirname(filepath)
         if not unipath.exists(base):
             os.makedirs(pathof(base))
@@ -654,15 +792,18 @@ class Wrapper(object):
             fp.write(data)
         self.other.append(id)
         self.added.append(id)
-        self.id_to_filepath[id] = desired_path
+        self.book_href_to_filepath[id] = desired_path
 
     def deleteotherfile(self, book_href):
         id = unicode_str(book_href)
-        if id in self.id_to_href:
-            raise WrapperException('Incorrect interface routine - use deletefile')
-        filepath = self.id_to_filepath.get(id, None)
+        id = unquoteurl(id)
         if id is None:
-            raise WrapperException('book href does not exist')
+            raise WrapperException('None is not a valid book hrefbook href')
+        if id not in self.other and id in self.id_to_href:
+            raise WrapperException('Incorrect interface routine - use deletefile')
+        filepath = self.book_href_to_filepath.get(id, None)
+        if filepath is None:
+            raise WrapperException('Book href does not exist')
         if id in PROTECTED_FILES:
             raise WrapperException('attempt to delete protected file')
         add_to_deleted = True
@@ -680,7 +821,7 @@ class Wrapper(object):
                 del self.modified[id]
         if add_to_deleted:
             self.deleted.append(('other', id, book_href))
-        del self.id_to_filepath[id]
+        del self.book_href_to_filepath[id]
 
 
     # utility routine to copy entire ebook to a destination directory
@@ -693,10 +834,18 @@ class Wrapper(object):
         for id in self.id_to_filepath:
             rpath = self.id_to_filepath[id]
             in_manifest = id in self.id_to_mime
-            if in_manifest:
-                data = self.readfile(id)
-            else:
-                data = self.readotherfile(id)
+            data = self.readfile(id)
+            filepath = os.path.join(destdir,rpath)
+            base = os.path.dirname(filepath)
+            if not unipath.exists(base):
+                os.makedirs(base)
+            if isinstance(data,text_type):
+                data = utf8_str(data)
+            with open(pathof(filepath),'wb') as fp:
+                fp.write(data)
+        for id in self.book_href_to_filepath:
+            rpath = self.book_href_to_filepath[id]
+            data = self.readotherfile(id)
             filepath = os.path.join(destdir,rpath)
             base = os.path.dirname(filepath)
             if not unipath.exists(base):
@@ -716,42 +865,11 @@ class Wrapper(object):
             apaths.append(unipath.abspath(os.path.join(self.usrsupdir,"hunspell_dictionaries")))
         else:
             # Linux
-            system_hunspell_dicts = ''
-            share_prefix = ''
-            share_override = ''
-
-            # get the env var SIGIL_DICTIONARIES set at launch time.
-            if 'SIGIL_DICTIONARIES' in os.environ.keys():
-                system_hunspell_dicts = os.environ['SIGIL_DICTIONARIES']
-
-            # Runtime env var override of 'share/sigil' directory.
-            if 'SIGIL_EXTRA_ROOT' in os.environ.keys():
-                share_override = os.environ['SIGIL_EXTRA_ROOT']
-
-            # The sigil launch script in <install_prefix>/bin knows where Sigil's build time
-            # share prefix is and sets the env var SIGIL_SHARE_PREFIX to its value.
-            if 'SIGIL_SHARE_PREFIX' in os.environ.keys():
-                share_prefix = os.environ['SIGIL_SHARE_PREFIX']
-
-            # If someone didn't launch Sigil with its launch script, this may save the
-            # day (as long as the user didn't override SHARE_INSTALL_PREFIX at compile time).
-            if not len(share_prefix) and not len(share_override):
-                share_prefix = unipath.abspath(os.path.join(self.appdir,"..",".."))
-
-            # If the SIGIL_DICTIONARIES env var has content, use it for the dictionary location.
-            if len(system_hunspell_dicts):
-                for path in system_hunspell_dicts.split(':'):
-                    apaths.append(unipath.abspath(path.strip()))
-            else:
-                # If the 'share/sigil' location has indeed been overridden at runtime, use that.
-                if len(share_override):
-                    apaths.append(unipath.abspath(os.path.join(share_override, "hunspell_dictionaries")))
-                else:
-                    # Otherwise, use Sigil's bundled hunspell dictionary location.
-                    apaths.append(unipath.abspath(os.path.join(share_prefix, "share", 'sigil', "hunspell_dictionaries")))
+            for path in self.linux_hunspell_dict_dirs:
+                apaths.append(unipath.abspath(path.strip()))
             apaths.append(unipath.abspath(os.path.join(self.usrsupdir,"hunspell_dictionaries")))
         return apaths
-                        
+
     def get_gumbo_path(self):
         if sys.platform.startswith('darwin'):
             lib_dir = unipath.abspath(os.path.join(self.appdir,"..","lib"))
@@ -763,7 +881,7 @@ class Wrapper(object):
             lib_dir = unipath.abspath(self.appdir)
             lib_name = 'libsigilgumbo.so'
         return os.path.join(lib_dir, lib_name)
-            
+
     def get_hunspell_path(self):
         if sys.platform.startswith('darwin'):
             lib_dir = unipath.abspath(os.path.join(self.appdir,"..","lib"))
@@ -775,4 +893,3 @@ class Wrapper(object):
             lib_dir = unipath.abspath(self.appdir)
             lib_name = 'libhunspell.so'
         return os.path.join(lib_dir, lib_name)
-
